

/*
 * ConnectionManager.java
 *
 * Created on 9 de Maio de 2006, 17:06
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package br.ufsc.das.blocking;

import br.ufsc.das.util.Configuration;
import br.ufsc.das.util.EventListener;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.net.Socket;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.Semaphore;
import java.util.concurrent.TimeUnit;

/**
 *
 * @author edualchieri
 */
public class ConnectionManager {
    
    private Map<Integer,SynchronizedOutput> connectionsOut = new Hashtable<Integer,SynchronizedOutput>();
    private EventListener listener;
    
    private Semaphore sm = new Semaphore(1);
    
    private ServerThreadPool serverPool;
    
    private Configuration config;
    
    private List<Integer> disconnected = new LinkedList<Integer>();
    
    /** Creates a new instance of ConnectionManager */
    public ConnectionManager(Configuration config) {
        this.config = config;
    }
    
    public EventListener getListener(){
        return this.listener;
    }
    
    public void setListener(EventListener listener){
        this.listener = listener;
        serverPool = new ServerThreadPool(20,50,50, 
                TimeUnit.SECONDS,new LinkedBlockingQueue(),
                listener,this);
        
        int autoConnect = config.getAutoConnectLimit();
        int me = config.getProcessId();
        for(int i = 0; i < autoConnect && i < me; i++){
            this.getConnectionOutput(i);
        }
    }
    
    public ServerThreadPool getServerPool(){
        return this.serverPool;
    }
    
    public SynchronizedOutput acceptConnection(Socket s, int from) throws Exception{
        DataOutputStream out = new DataOutputStream(s.getOutputStream());
        SynchronizedOutput so = new SynchronizedOutput(out,this,from);
        this.connectionsOut.put(from, so);
        String name = s.getInetAddress().getCanonicalHostName();
        System.out.println("Recebeu conexao de "+name);
        return so;
    }
    
    private SynchronizedOutput openConnection(int id) throws Exception{
        Socket s = null;
        boolean opened = false;
        while(!opened){
            try{
                ////br.ufsc.das.util.Logger.println("Conectando com: "+Configuration.getHost(id)+":"+Configuration.getPort(id));
                s = new Socket(config.getHost(id),config.getPort(id));
                String name = s.getInetAddress().getCanonicalHostName();
                System.out.println("Conected to "+name);
                s.setTcpNoDelay(true);
                opened = true;
            }catch(Exception e){
                synchronized(this){
                    try{
                        this.wait(1000);
                    }catch(InterruptedException ioe){
                        ioe.printStackTrace(System.out);
                    }
                }
            }
        }
        DataOutputStream out = new DataOutputStream(s.getOutputStream());
        DataInputStream in = new DataInputStream(s.getInputStream());
        out.writeInt(config.getProcessId());
        out.flush();
        SynchronizedOutput so = new SynchronizedOutput(out,this,id);
        //if(Configuration.getProcessId() >= Configuration.getN()){
        serverPool.execute(in,id,so);
        //}
        return so;
    }
    
   /* public void removeConnectionOutput(int id){
        this.connectionsOut.remove(id);
    }*/
    
    public void disconnected(int target){
        this.disconnected.add(new Integer(target));
    }
    
    public SynchronizedOutput getConnectionOutput(int id){
        if(disconnected.contains(new Integer(id))){
            return null;
        }
        
        SynchronizedOutput ret =  this.connectionsOut.get(id);
        if(ret == null){
            try{
                this.sm.acquire();
            }catch(InterruptedException ie){
                ie.printStackTrace(System.out);
            }
            
            ret =  this.connectionsOut.get(id);
            if(ret == null){
                try{
                    ret = openConnection(id);
                }catch(Exception e){
                    e.printStackTrace(System.out);
                }
                this.connectionsOut.put(id,ret);
            }
            this.sm.release();
        }
        return ret;
    }
}
