

package br.ufsc.das.usage;

import br.ufsc.das.usage.nio.Connector;
import br.ufsc.das.usage.nio.PacketChannel;
import br.ufsc.das.usage.nio.ProtocolDecoderAuth;
import br.ufsc.das.usage.nio.ReceivedPacket;
import br.ufsc.das.usage.nio.SelectorThread;
import br.ufsc.das.usage.nio.ProtocolDecoder;
import br.ufsc.das.util.EventListener;
import br.ufsc.das.util.ReceivedMessage;
import br.ufsc.das.util.Configuration;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.SocketChannel;
import java.security.KeyFactory;
import java.security.PublicKey;
import java.security.spec.X509EncodedKeySpec;
import java.util.Arrays;
import java.util.concurrent.Semaphore;
import javax.crypto.KeyAgreement;
import javax.crypto.Mac;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;


/*
 * ChannelWriter.java
 *
 * Created on 27 de Janeiro de 2006, 15:33
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

/**
 *
 * @author alchieri
 */
public class SendReceiveAuthPacket extends SendReceivePacket {
   
    private Mac mac;
    private Semaphore macSem = new Semaphore(1);
    
    /**
     * Used to connect to target
     */
    public SendReceiveAuthPacket(int target, EventListener l,
            SelectorThread st, ProcessWriter c, Configuration conf){
        super(target,l,st,c,conf);
    }
    
    /**
     * Used to receive connections
     */
    public SendReceiveAuthPacket(PacketChannel channel,EventListener l, 
            ProcessWriter c, Configuration conf){
        super(channel,l,c,conf);
        this.sm = new Semaphore(0);
        this.connectionEstablished = false;
    }
    
    
    @Override
    public void write(byte[] msg){
        //br.ufsc.das.util.Logger.println("(SendReceiveAuthPacket.write) Waiting for a semaphore to be released");
        try{
            this.sm.acquire();
        }catch(Exception e){
            e.printStackTrace();
        }
        //br.ufsc.das.util.Logger.println("(SendReceiveAuthPacket.write) Semaphore acquired");
        sendData(msg, getAuthenticationCode(msg), ProtocolDecoderAuth.STX);
    }
    
    private final byte[] getAuthenticationCode(byte[] bytesToSend){
         try{
            this.macSem.acquire();
            byte[] ret = mac.doFinal(bytesToSend);
            this.macSem.release();
            return ret;
        }catch(Exception e){
            e.printStackTrace();
            return null;
        }
    }
    
    private void sendData(byte[] data, byte[] aut,byte ctr){
        ByteBuffer buffer = ByteBuffer.allocate(data.length+aut.length+9); // 9 = 2*int + ctr
        buffer.put(ctr);
        buffer.putInt(data.length);
        buffer.putInt(aut.length);
        buffer.put(data);
        buffer.put(aut);
        buffer.limit(buffer.position());
        buffer.flip();
        this.channel.sendPacket(buffer);
    }
    
    /**
     * A new client connected. Creates a PacketChannel to handle it.
     */
    @Override
    public void connectionEstablished(Connector connector, SocketChannel sc) {
        try {
            sc.socket().setReceiveBufferSize(200*1024);
            sc.socket().setSendBufferSize(200*1024);
            
            channel = new PacketChannel(sc,st,new ProtocolDecoderAuth(),this);
            channel.resumeReading();
            
            sendData(new Integer(conf.getProcessId()).toString().getBytes(),
                    this.clientSide.getDHPubKey().getEncoded(),
                    ProtocolDecoderAuth.SATX);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    
    private final boolean checkAuth(ReceivedPacket pckt){
        return Arrays.equals(getAuthenticationCode(pckt.getData()),
                pckt.getAuthenticationCode());
    }
    
    @Override
    public void packetArrived(PacketChannel pc, ReceivedPacket pckt) {
        ////br.ufsc.das.util.Logger.println("[" + pc.toString() + "] Packet received. Size: "+pckt.getData().length);
        if(pckt.getCtr() == ProtocolDecoder.STX){
            if(checkAuth(pckt)){
                listener.deliver(new ReceivedMessage(pckt.getData(),this.remoteId));
            }else{
                System.err.println("Autentication failure in message from "+remoteId);
            }
        }else{
            int from = Integer.decode(new String(pckt.getData()));
            this.remoteId = from;
            String name = pc.getSocketChannel().socket().getInetAddress().getCanonicalHostName();
            clientSide.put(from,this);
            if(conf.isHostSetted(from)){// esta em hosts...
                String ip = pc.getSocketChannel().socket().getInetAddress().getHostAddress();
                if(!checkServer(from,name, ip)){
                    System.err.println("Unexpected connection attempt from "+ip);
                    pc.close();
                    clientSide.remove(from);
                    return;
                }
            }

            if(pckt.getCtr() == ProtocolDecoderAuth.SATX){
                //br.ufsc.das.util.Logger.println("Connection received from "+name);
                this.exchangeSecret(pckt.getAuthenticationCode());
                byte[] pId = new Integer(conf.getProcessId()).toString().getBytes();
                sendData(pId,this.clientSide.getDHPubKey().getEncoded(),ProtocolDecoderAuth.RATX);
            }else{
               System.out.println("Conected to "+name);
               this.exchangeSecret(pckt.getAuthenticationCode());
            }

            connectionEstablished = true;
            this.clientSide.connectionEstablished(this.remoteId, this);
        }

        try {
            pc.resumeReading();
        } catch (Exception e) {
            e.printStackTrace(System.out);
        }
    }
    
    private void exchangeSecret(byte[] otherPartyPubKey){
         try {
            X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(otherPartyPubKey);
            KeyFactory keyFact = KeyFactory.getInstance("DH");
            PublicKey publicKey = keyFact.generatePublic(x509KeySpec);
            KeyAgreement ka = KeyAgreement.getInstance("DH");
            ka.init(this.clientSide.getDHPriKey());
            ka.doPhase(publicKey, true);
            byte[] secret = ka.generateSecret();
            SecretKey secretKey = new SecretKeySpec(secret,conf.getHmacAlgorithm());
            this.mac = Mac.getInstance(secretKey.getAlgorithm());
            this.mac.init(secretKey);
        } catch(Exception e){
            e.printStackTrace();
        }
    }
}
