package br.ufsc.das.usage;

import br.ufsc.das.usage.nio.Connector;
import br.ufsc.das.usage.nio.PacketChannel;
import br.ufsc.das.usage.nio.ReceivedPacket;
import br.ufsc.das.usage.nio.SelectorThread;
import br.ufsc.das.usage.nio.ProtocolDecoder;
import br.ufsc.das.util.EventListener;
import br.ufsc.das.util.ReceivedMessage;
import br.ufsc.das.util.Configuration;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.SocketChannel;
import java.util.concurrent.Semaphore;


/*
 * ChannelWriter.java
 *
 * Created on 27 de Janeiro de 2006, 15:33
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

/**
 *
 * @author alchieri
 */
public class SendReceivePacket {

    protected SelectorThread st;
    protected PacketChannel channel;
    protected EventListener listener;
    protected boolean connectionEstablished = false;
    protected Semaphore sm;
    protected int remoteId = -1;
    protected ProcessWriter clientSide;
    protected Configuration conf;  

    /** 
     * Used to connect to target
     */
    public SendReceivePacket(int target, EventListener l,
            SelectorThread st, ProcessWriter c, Configuration conf){
        this.st = st;
        this.listener = l;
        this.remoteId = target;
        this.conf = conf;

        //the semaphore begin locked and will be unlocked after first message
        //is received and the connection is setted
        this.sm = new Semaphore(0);

        this.clientSide = c;
    }
    
    /**
     * Used to receive connections
     */
    public SendReceivePacket(PacketChannel channel, EventListener l,
            ProcessWriter c, Configuration conf){
        this.sm = new Semaphore(1);
        this.listener = l;
        this.channel = channel;
        this.connectionEstablished = true;
        this.clientSide = c;
        this.conf = conf;
    }

    public int getRemoteId() {
        return this.remoteId;
    }

    public void connect()  throws Exception {
        InetSocketAddress addr = conf.getRemoteAddress(this.remoteId);
        new Connector(this.st, addr, this).connect();
    }
    
    public boolean isConnected() {
        return this.connectionEstablished;
    }
    
    public void setConnected() {
        this.connectionEstablished = true;
    }
    
    public void write(byte[] msg) {
        //br.ufsc.das.util.Logger.println("(SendReceivePacket.write) Waiting for a semaphore to be released");
        try{
            this.sm.acquire();
        }catch(Exception e){
            e.printStackTrace();
        }
        //br.ufsc.das.util.Logger.println("(SendReceivePacket.write) Semaphore acquired");
        sendData(msg,ProtocolDecoder.STX);
    }
    
    private final void sendData(byte[] data ,byte ctr){
        int dataLength = data.length;
        ByteBuffer buffer = ByteBuffer.allocate(dataLength+5); // 5 = 1*int + ctr
        buffer.put(ctr);
        buffer.putInt(dataLength);
        buffer.put(data);
        //buffer.limit(buffer.position());
        buffer.flip();
        this.channel.sendPacket(buffer);
    }
    
    /**
     * A new client connected.
     * Creates a PacketChannel to handle it.
     */
    public void connectionEstablished(Connector connector, SocketChannel sc) {
        try {
            sc.socket().setReceiveBufferSize(200*1024);
            sc.socket().setSendBufferSize(200*1024);

            this.channel = new PacketChannel(sc,st,new ProtocolDecoder(),this);
            this.channel.resumeReading();

            //send message to initiate connection
            sendData(new Integer(conf.getProcessId()).toString().getBytes(),
                    ProtocolDecoder.ATX);
            
            System.out.println("Connected to "+
                    sc.socket().getInetAddress().getHostName());

            this.connectionEstablished = true;
            this.clientSide.connectionEstablished(this.remoteId, this);
          } catch (IOException e) {
            e.printStackTrace();
        }
    }
    
    
    
    public void connectionFailed(Connector connector, Exception cause) {
        this.clientSide.connectionFailed(this.remoteId, this);
    }
    
    
    public void packetArrived(PacketChannel pc, ReceivedPacket pckt) {
        ////br.ufsc.das.util.Logger.println("[" + pc.toString() + "] Packet received. Size = "+pckt.getData().length);

        if(pckt.getCtr() == ProtocolDecoder.STX) {
             listener.deliver( new ReceivedMessage(pckt.getData(),this.remoteId));
        } else {
            int from = Integer.decode(new String(pckt.getData()));
            this.remoteId = from;
            clientSide.put(from,this);

            if(conf.isHostSetted(from)){ //esta em hosts...
                String name = pc.getSocketChannel().socket().getInetAddress().getHostName();
                String ip = pc.getSocketChannel().socket().getInetAddress().getHostAddress();
                if(!checkServer(from, name, ip)){
                    System.err.println("Unexpected connection attempt from "+ip);
                    pc.close();
                    clientSide.remove(from);
                    return;
                }
            }
        }
        
        try {
            pc.resumeReading();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }
    }
    
    protected final boolean checkServer(int from, String name, String ip){
        String localConf = conf.getHost(from);
        if(!(name.equalsIgnoreCase(localConf) || ip.equalsIgnoreCase(localConf))){
            int i = name.indexOf(".");
            if(!name.substring(0,i).equalsIgnoreCase(localConf)){
                return false;
            }
        }
        return true;
    }
    
    public void socketException(PacketChannel pc, Exception ex) {
        //br.ufsc.das.util.Logger.println("[" + remoteId + "] Error: " + ex.getMessage());
        this.sm.release();
    }
    
    public void socketDisconnected(PacketChannel pc) {
        this.clientSide.remove(remoteId);
        String name = pc.getSocketChannel().socket().getInetAddress().getCanonicalHostName();
        //br.ufsc.das.util.Logger.println("[" + name  + "("+remoteId+")] Disconnected.");
    }
    
    
    public final void packetSent() {
        this.sm.release();
        //br.ufsc.das.util.Logger.println("(SendReceivePacket.packetSent) Semaphore released");
    }
}
