/*
 * Acceptor.java
 *
 * Created on 24 de Maio de 2006, 11:29
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package br.ufsc.das.usage.nio;



import br.ufsc.das.usage.ConnectionListener;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.nio.channels.*;


final public class Acceptor{

    private ServerSocketChannel ssc;

    private final SelectorThread ioThread;

    private final int listenPort;

    private final ConnectionListener listener;
    
    public Acceptor(
            int listenPort,
            SelectorThread ioThread,
            ConnectionListener listener) {
        this.ioThread = ioThread;
        this.listenPort = listenPort;
        this.listener = listener;
    }
    

    public void openServerSocket() throws IOException {
        ssc = ServerSocketChannel.open();
        InetSocketAddress isa = new InetSocketAddress(listenPort);
        ssc.socket().bind(isa, 100);
        
        ioThread.registerChannelLater(ssc,
                SelectionKey.OP_ACCEPT,
                this,
                new CallbackErrorHandler() {
            public void handleError(Exception ex) { 
                listener.socketError(Acceptor.this, ex);
            }
        });
    }
    
    public String toString() {
        return "ListenPort: " + listenPort;
    }
    
    public void handleAccept() {
        SocketChannel sc = null;
        try {
            sc = ssc.accept();
            Socket s = sc.socket();
            ioThread.addChannelInterestNow(ssc, SelectionKey.OP_ACCEPT);
        } catch (IOException e) {
            listener.socketError(this, e);
        }
        if (sc != null) {
            listener.socketConnected(this, sc);
        }
    }
    
    public void close()  {
        try {
            ioThread.invokeAndWait(new Runnable() {
                public void run() {
                    if (ssc != null) {
                        try {
                            ssc.close();
                        } catch (IOException e) {
                            // Ignore
                        }
                    }
                }
            });
        } catch (InterruptedException e) {
            // Ignore
        }
    }
}