/*
 * Connector.java
 *
 * Created on 24 de Maio de 2006, 11:32
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package br.ufsc.das.usage.nio;

import br.ufsc.das.usage.SendReceivePacket;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.channels.SelectionKey;
import java.nio.channels.SocketChannel;


final public class Connector{
    
    private SocketChannel sc;
    
    private final InetSocketAddress remoteAddress;
    
    private final SelectorThread selectorThread;
    
    private final SendReceivePacket listener;
    
    
    public Connector(SelectorThread selector,
            InetSocketAddress remoteAddress,
            SendReceivePacket listener){
        this.selectorThread = selector;
        this.remoteAddress = remoteAddress;
        this.listener = listener;
    }
    
    public void connect() throws IOException {
        sc = SocketChannel.open();
        sc.configureBlocking(false);
        sc.connect(remoteAddress);
        selectorThread.registerChannelLater(sc, SelectionKey.OP_CONNECT, this,
                new CallbackErrorHandler() {
                    public void handleError(Exception ex) {
                        listener.connectionFailed(Connector.this, ex);
                    }
                });
    }
    

    public void handleConnect() {
        try {
            if (!sc.finishConnect()) {
                listener.connectionFailed(this, null);
                return;
            }
            listener.connectionEstablished(this, sc);
        } catch (IOException ex) {
            listener.connectionFailed(this, ex);
        }
    }
    
    public String toString() {
        return "Remote endpoint: " +
                remoteAddress.getAddress().getHostAddress() + ":" + remoteAddress.getPort();
    }
}